@@ -1,19 +1,27 @@
 #[macro_use]
 extern crate clap;
-extern crate crossbeam_channel as channel;
-extern crate ignore;
+extern crate rayon;
 extern crate unicode_width;
-extern crate walkdir;
+
+use std::collections::HashSet;
 
 use self::display::draw_it;
-use crate::utils::is_a_parent_of;
 use clap::{App, AppSettings, Arg};
+use dir_walker::walk_it;
+use dir_walker::WalkData;
+use filter::{get_biggest, get_by_depth};
 use std::cmp::max;
 use std::path::PathBuf;
 use terminal_size::{terminal_size, Height, Width};
-use utils::{find_big_ones, get_dir_tree, simplify_dir_names, sort, Node};
+use utils::get_filesystem_devices;
+use utils::simplify_dir_names;
 
+mod dir_walker;
 mod display;
+mod display_node;
+mod filter;
+mod node;
+mod platform;
 mod utils;
 
 static DEFAULT_NUMBER_OF_LINES: usize = 30;
@@ -75,7 +83,8 @@ fn main() {
                 .short("d")
                 .long("depth")
                 .help("Depth to show")
-                .takes_value(true),
+                .takes_value(true)
+                .conflicts_with("number_of_lines"),
         )
         .arg(
             Arg::with_name("number_of_lines")
@@ -140,7 +149,7 @@ fn main() {
             Arg::with_name("ignore_hidden")
                 .short("i") // Do not use 'h' this is used by 'help'
                 .long("ignore_hidden")
-                .help("Obey .git_ignore rules & Do not display hidden files"),
+                .help("Do not display hidden files"),
         )
         .arg(
             Arg::with_name("width")
@@ -150,7 +159,6 @@ fn main() {
                 .number_of_values(1)
                 .help("Specify width of output overriding the auto detection of terminal width"),
         )
-
         .arg(Arg::with_name("inputs").multiple(true))
         .get_matches();
 
@@ -181,38 +189,48 @@ fn main() {
             .map_err(|_| eprintln!("Ignoring bad value for depth"))
             .ok()
     });
-    if options.is_present("depth") && number_of_lines != default_height {
-        eprintln!("Use either -n or -d. Not both");
-        return;
-    }
 
     let no_colors = init_color(options.is_present("no_colors"));
     let use_apparent_size = options.is_present("display_apparent_size");
-    let limit_filesystem = options.is_present("limit_filesystem");
-    let ignore_directories = match options.values_of("ignore_directory") {
-        Some(i) => Some(i.map(PathBuf::from).collect()),
-        None => None,
-    };
+    let ignore_directories: Vec<PathBuf> = options
+        .values_of("ignore_directory")
+        .map(|i| i.map(PathBuf::from).collect())
+        .unwrap_or_default();
+
     let by_filecount = options.is_present("by_filecount");
-    let show_hidden = !options.is_present("ignore_hidden");
+    let ignore_hidden = options.is_present("ignore_hidden");
+    let limit_filesystem = options.is_present("limit_filesystem");
 
     let simplified_dirs = simplify_dir_names(target_dirs);
-    let (errors, nodes) = get_dir_tree(
-        &simplified_dirs,
-        &ignore_directories,
+    let allowed_filesystems = {
+        if limit_filesystem {
+            get_filesystem_devices(simplified_dirs.iter())
+        } else {
+            HashSet::new()
+        }
+    };
+
+    let ignored_full_path: HashSet<PathBuf> = ignore_directories
+        .into_iter()
+        .flat_map(|x| simplified_dirs.iter().map(move |d| d.join(x.clone())))
+        .collect();
+
+    let walk_data = WalkData {
+        ignore_directories: ignored_full_path,
+        allowed_filesystems,
         use_apparent_size,
-        limit_filesystem,
         by_filecount,
-        show_hidden,
-    );
-    let sorted_data = sort(nodes);
-    let biggest_ones = {
+        ignore_hidden,
+    };
+
+    let (nodes, errors) = walk_it(simplified_dirs, walk_data);
+
+    let tree = {
         match depth {
-            None => find_big_ones(sorted_data, number_of_lines),
-            Some(_) => sorted_data,
+            None => get_biggest(nodes, number_of_lines),
+            Some(depth) => get_by_depth(nodes, depth),
         }
     };
-    let tree = build_tree(biggest_ones, depth);
 
     draw_it(
         errors,
@@ -225,35 +243,3 @@ fn main() {
         tree,
     );
 }
-
-fn build_tree(biggest_ones: Vec<(PathBuf, u64)>, depth: Option<usize>) -> Node {
-    let mut top_parent = Node::default();
-
-    // assume sorted order
-    for b in biggest_ones {
-        let n = Node {
-            name: b.0,
-            size: b.1,
-            children: Vec::default(),
-        };
-        recursively_build_tree(&mut top_parent, n, depth);
-    }
-    top_parent
-}
-
-fn recursively_build_tree(parent_node: &mut Node, new_node: Node, depth: Option<usize>) {
-    let new_depth = match depth {
-        None => None,
-        Some(0) => return,
-        Some(d) => Some(d - 1),
-    };
-    if let Some(c) = parent_node
-        .children
-        .iter_mut()
-        .find(|c| is_a_parent_of(&c.name, &new_node.name))
-    {
-        recursively_build_tree(c, new_node, new_depth);
-    } else {
-        parent_node.children.push(new_node);
-    }
-}
